package com.jiuqi.util;

import java.math.BigInteger;
import java.util.Random;

public final class Guid implements Cloneable
{
    private static final char[] HEX_CHARS;
    private static final char[] BASE64_CHARS;
    public static int GUID_SIZE;
    public static String emptyGuid;
    private static Random rndGuid;
    private byte[] data;
    
    static {
        HEX_CHARS = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        BASE64_CHARS = new char[] { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '=' };
        Guid.GUID_SIZE = 32;
        Guid.emptyGuid = "00000000000000000000000000000000";
        Guid.rndGuid = new Random(Math.round(Math.random() * 10000.0));
    }
    
    public Guid() {
        this.data = new byte[16];
    }
    
    public static String newGuid() {
        final Guid T = new Guid();
        T.generate();
        return T.toString();
    }
    
    public void generate() {
        final BigInteger BI1 = BigInteger.valueOf(System.currentTimeMillis());
        final BigInteger BI2;
        synchronized (Guid.rndGuid) {
            BI2 = new BigInteger(64, Guid.rndGuid);
            monitorexit(Guid.rndGuid);
        }
        byte[] BA = BI1.toByteArray();
        for (int i = 0; i < 8 - BA.length; ++i) {
            this.data[i] = 0;
        }
        for (int i = 0; i < BA.length; ++i) {
            this.data[8 - BA.length + i] = BA[i];
        }
        BA = BI2.toByteArray();
        for (int i = 0; i < 8 - BA.length; ++i) {
            this.data[8 + i] = 0;
        }
        for (int i = 0; i < BA.length; ++i) {
            if (BA.length - i <= 8) {
                this.data[16 - BA.length + i] = BA[i];
            }
        }
    }
    
    public Object clone() {
        final Guid copy = new Guid();
        copy.data = this.data.clone();
        return copy;
    }
    
    @Override
    public boolean equals(final Object o) {
        return o instanceof Guid && this.data.equals(((Guid)o).data);
    }
    
    @Override
    public String toString() {
        final StringBuffer str = new StringBuffer(32);
        for (int i = 0; i < this.data.length; ++i) {
            int code = this.data[i] >> 4 & 0xF;
            str.append(Guid.HEX_CHARS[code]);
            code = (this.data[i] & 0xF);
            str.append(Guid.HEX_CHARS[code]);
        }
        return str.toString();
    }
    
    public String tobase64() {
        final StringBuffer s = new StringBuffer("0000000000000000000000");
        int p = 0;
        int i = 0;
        while (i < 15) {
            int t;
            int b = t = this.data[i++];
            b = (b >>> 2 & 0x3F);
            s.setCharAt(p++, Guid.BASE64_CHARS[b]);
            b = (t = this.data[i++]);
            b = (b >>> 4 & 0xF);
            t &= 0x3;
            t <<= 4;
            b |= t;
            s.setCharAt(p++, Guid.BASE64_CHARS[b]);
            b = (t = this.data[i++]);
            b = (b >>> 6 & 0x3);
            t &= 0xF;
            t <<= 2;
            b |= t;
            s.setCharAt(p++, Guid.BASE64_CHARS[b]);
            t = (b = (t & 0x3F));
            s.setCharAt(p++, Guid.BASE64_CHARS[b]);
        }
        int t;
        int b = t = this.data[i++];
        b = (b >>> 2 & 0x3F);
        s.setCharAt(p++, Guid.BASE64_CHARS[b]);
        b = (t = 0);
        b = (b >>> 4 & 0xF);
        t &= 0x3;
        t <<= 4;
        b |= t;
        s.setCharAt(p++, Guid.BASE64_CHARS[b]);
        return new String(s);
    }
}
